package ca.ualberta.cs.poker.free.tournament;

import java.io.BufferedWriter;
import java.io.File;
import java.io.IOException;

/**
 * Main Tournament class. Runs the tournament but uses a few assumptions:
 * 
 * 1. For the remote launching of clients to work, you need to have ssh keys set
 * to allow commands to be run on the target machines, without prompt. That means
 * generating a keyset and loggin in at least once to avoid the prompt about
 * accepting the key.
 * 
 * 2. You also need to have directories with the necessary client files, called
 * comp1, comp2, ....  compN  so the clients can be copied.
 * 
 * 3. You would also need to have the proper user designated, as below, instead of
 * the users and IPs I have been using. 
 * 
 * If these things are done, the tree upon launch will run the entire tournament,
 * generate logfiles of the results and declare a winner.
 * 
 * @author Christian Smith
 *
 */

public class Tournament {
	
	public static void main(String[] args) {
				
		//right number of arguments?
		if ( args.length != 3 ) {
			System.out.println( "\nERROR: Incorrect number of arguments\n\n");
			printUsage();
		}
		
		int timePerHand = Integer.parseInt( args[0] );
		int competitors = Integer.parseInt( args[1]);
		int matches = Integer.parseInt( args[2]);
		
		System.out.println( timePerHand + " " + competitors + " " + matches );
		
		// sanity check
		if ( (competitors < 0 || competitors > 100) && (matches < 0 || matches > 100) ) {
			System.out.println("Please make sure numCompetitors and numMatches is reasonable. ie 0 < num < 100");
			System.exit(0);
		}
		
		//runSingleElimination( matches )
		//runFourCompetitors();
		//runTestMatch();
		runRoundRobin( timePerHand, competitors, matches  );
		
		
	}
	
public static  void printUsage() {
		
		System.out.println( "\nUsage: " + "java ca.ualberta.cs.poker.free.server.Tournament <millisecondsPerHand> <numberOfCompetitors> <numberOfMatches>");
		System.out.println( "numCompetitors: number of players");
		System.out.println( "numMatches: \tnumber of matches per round of play");	    
	    System.exit(0);
	    		
		
	}

	
	/*
	 * runSingleElimination needs to be set up by hand for the number of 
	 * competitors
	 * 
	 */
	public void runSingleElimination( int numberOfMatches) {
		
		int numMatches = numberOfMatches;
		
		//runFourCompetitors();
		
		
		// create tree structure with nodes
		Competitor one = new Competitor( "comp1" );
		Competitor two = new Competitor( "comp2" );
		
		Series A = new Series( 1, numMatches, one, two );
		
		Competitor three = new Competitor( "comp3" );
		Competitor four = new Competitor( "comp4" );
		
		Series B = new Series( 2, numMatches, three, four );
		
		Competitor five = new Competitor( "comp5" );
		Competitor six = new Competitor( "comp6" );
		
		Series C = new Series( 3, numMatches, five, six );
		
		Series D = new Series( 4, numMatches, A, B );
		Series E = new Series( 5, numMatches, D, C );
		
		Series root = E;

		
		// make the directory to store results
		String directory = "data/results/";
		new File(directory).mkdirs(); 
		
		Forge f = new Forge();
		f.addIPToAvailable("129.128.23.142");
		f.addIPToAvailable("129.128.23.106");
		
		int totalNumMatches = 0;
		
		// initial load to check for completeness
		root.load( f );
		
		while ( root.complete() != true ) {
			
			// start the tourney
			root.load( f );
			
			System.out.println( f.hasNumberOfMatches() + " matches queued. " + f.hasNumberOfIPs() + " IPs");
			
			// there must be two machines to run the match
			if ( f.isReady() ) {
				
				Match m = f.getFirstMatch();
				
				System.out.println( "Running Threaded Match: " + m.toString());
				
				m.run();
				totalNumMatches++;
				
			}
			
			// sleep and check again
			try {
				System.out.println( "Waiting...");
				Thread.currentThread().sleep(3000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				System.err.println( "InteruptedException while waiting for matches in Tournament");
				e.printStackTrace();
			}
			
		}
		
		System.out.println( "Tournament Complete, the winner is: " + root.getWinner() 
				+ " after running " + totalNumMatches + " matches");
		
		
	}
	
	public static void runTestMatch () {
		String directory = "data/results/";
		new File(directory).mkdirs(); // make the directory to store results
		
		Forge f = new Forge();
		f.addIPToAvailable("129.128.23.142");
		f.addIPToAvailable("129.128.23.106");
		
		Match test1 = new Match( "comp1", "comp2", "129.128.23.142", "129.128.23.106", 1, 1, "RR", 7000);
	
		test1.run();
		System.exit(0);
	}
	
	public static void runFourCompetitors( ) {

		int numMatches = 2;
		Forge f = new Forge();
		f.addIPToAvailable("129.128.23.142");
		f.addIPToAvailable("129.128.23.106");
		
		
		//		 create tree structure with nodes
		Competitor one = new Competitor( "comp1" );
		Competitor two = new Competitor( "comp2" );
		
		Series A = new Series( 1, numMatches, one, two );
		
		Competitor three = new Competitor( "comp3" );
		Competitor four = new Competitor( "comp4" );		
		
		Series B = new Series( 2, numMatches, three, four );
		Series C = new Series( 3, numMatches, A, B );
		
		Series root = C;
		
		root.load( f );
		
		while ( root.complete() != true ) {
			
			System.out.println( f.hasNumberOfMatches() + " matches queued. " + f.hasNumberOfIPs() + " IPs");
			
			// start the tourney
			root.load( f );
			
			// there must be two machines to run the match
			if ( f.isReady() ) {
				System.out.println( "Good to go");
				
				Match m = f.getFirstMatch();
				m.run();
				
				f.addIPToAvailable( m.getIp1() );
				f.addIPToAvailable( m.getIp2() );
				
			}
	
			
			// sleep and check again
			try {
				System.out.println( "Waiting...");
				Thread.currentThread().sleep(3000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				System.err.println( "InteruptedException while waiting for matches in Tournament");
				e.printStackTrace();
			}
			
		}
		
		System.out.println( "Tournament Complete, the winner is: " + root.getWinner() );
	
	}
	
	public static void runRoundRobin( int msPerHand, int numberOfCompetitors, int numberOfMatches ) {
				
		RoundRobinSeries rr = new RoundRobinSeries(numberOfCompetitors, numberOfMatches, msPerHand);
		double totalNumMatches = 0;
		
		Forge f = new Forge();
		f.addIPToAvailable("uc02");
		f.addIPToAvailable("uc03");
		f.addIPToAvailable("uc04");
		f.addIPToAvailable("uc05");
		f.addIPToAvailable("uc06");
		f.addIPToAvailable("uc07");
		f.addIPToAvailable("uc08");
		f.addIPToAvailable("uc09");
		f.addIPToAvailable("uc10");
		f.addIPToAvailable("uc11");
		f.addIPToAvailable("uc12");
		f.addIPToAvailable("uc13");
		f.addIPToAvailable("uc14");
		f.addIPToAvailable("uc15");
		f.addIPToAvailable("uc16");
		
		while ( rr.complete() != true ) {
			
			// start the tourney
			rr.load( f );
			
			System.out.println( f.hasNumberOfMatches() + " matches queued. " + f.hasNumberOfIPs() + " IPs");
			
			// there must be two machines to run the match
			if ( f.isReady() ) {
				
				Match m = f.getFirstMatch();
				
				System.out.println( "Running Threaded Match: " + m.toString());
				
				m.run();
				totalNumMatches++;
				
			}
			
			// sleep and check again
			try {
				System.out.println( "Waiting for matches to run...");
				Thread.currentThread().sleep(3000);
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				System.err.println( "InteruptedException while waiting for matches in Tournament");
				e.printStackTrace();
			}
			
		}	
		
		System.out.println( "Round Robin Tournament Complete, the winner is: " + rr.getWinner() 
				+ " after running " + totalNumMatches + " matches this run.");
				
	}
	
}